;###########################################################################
;    This file is part of KillingMachine game made by PizzarT.
;
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;
;###########################################################################

;###########################################################################
;#### FICHERO: main.s
;###########################################################################
;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

;; Include all CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "menu.h.s"
.include "wave.h.s"

;;
;; Start of _DATA area 
;;  SDCC requires at least _DATA and _CODE areas to be declared, but you may use
;;  any one of them for any purpose. Usually, compiler puts _DATA area contents
;;  right after _CODE area contents.
;;
.area _DATA
.area _CODE

m_game_over: .db 00
m_win: .db 00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RESETEA EL JUEGO ENTERO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_reset:
   ld    a, #0
   ld    (m_num_enemies), a
   ld    (m_num_enemyBullets), a
   ld    (m_num_heroBullets), a
   ld    (m_num_total_enemies), a
   ld    (m_t_nextWave), a

   ld    (m_game_over), a
   ld    (m_win), a

   ld    hl, #enemy_vector
   ld    (m_next_enemy), hl

   ld    hl, #enemyBullet_vector
   ld    (m_next_enemyBullet), hl

   ld    hl, #heroBullet_vector
   ld    (m_next_heroBullet), hl

   ld    ix, #hero_data
   ld    a, #12
   ld    ce_hp(ix), a

   ld    a, #0x27
   ld    de_x(ix), a
   ld    a, #0x64
   ld    de_y(ix), a


   ld    hl, #waves_vector
   ld    (m_next_wave), hl

   ld    a, #0
   ld    (m_num_wave), a

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; LIMPIA LOS DOS BUFFERS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_resetBuffers:
   call  rend_newScene
   call  ent_initDrawableEntityVector
   call  rend_newScene

   ret

;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;
_main::
   ld    sp, #0x8000   ;; Pila desde 7FFF hacia abajo

   ;; Disable firmware to prevent it from interfering with string drawing
   call  cpct_disableFirmware_asm

   ld    l, #16
   ld    h, #20
   call cpct_setPALColour_asm

   call  rend_initBackbuffer

   ld    c, #0
   call  cpct_setVideoMode_asm

   ld    hl, #_heroe_pal
   ld    de, #16
   call  cpct_setPalette_asm

   call  rend_drawMap

menu:
   call  menu_show

   call  ent_initHeroStats

loop:
   call  ent_initDrawableEntityVector

   call  wave_checkLoad

   ;; Comprobamos si hemos ganado
   ld    a, (m_win)
   sub   #1
   jr    nz, no_win
      ld    ix, #hero_data
      ld    d, ce_hp(ix)
      call  ent_removeHeartHeroStats
      call  rend_removeWaveNumber
      call  game_resetBuffers
      call  game_reset
      call  menu_win
      jr    menu

no_win:
   ld    ix, #hero_data
   call  ent_update
   call  rend_drawEntity
   call  ent_regeneraHealth

   call  ent_spawnearEnemy

   call  ent_updateEntities

   ;; Comprobamos si ha terminado el juego
   ld    a, (m_game_over)
   sub   #1
   jr    nz, continuar
      call  rend_removeWaveNumber
      call  game_resetBuffers
      call  game_reset
      call  menu_gameOver
      jr    menu

continuar:
   ld 	b, #10
   call  cpct_waitHalts_asm
   call  cpct_waitVSYNC_asm

   call  rend_newScene

   ;; Loop forever
   jr    loop
