;###########################################################################
;    This file is part of KillingMachine game made by PizzarT.
;
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;
;###########################################################################

;###########################################################################
;#### FICHERO: render.s
;###########################################################################

.include "render.h.s"
.include "main.h.s"

m_front_buffer: .db #0xC0
m_back_buffer:	 .db #0x80

k_zero_ascii = 48

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; LIMPIA EL BACKBUFFER Y CAMBIA DE BUFFER
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_newScene:
	call 	rend_switchBuffers
	call 	rend_clearBackbuffer
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INICIALIZA UN BUFFER
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_initBackbuffer:
	ld 	hl, #0x8000
	ld 	(hl), #0
	ld 	de, #0x8000 + 1
	ld 	bc, #0x4000 - 1
	ldir 
	
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; BORRA BACKBUFFER
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_clearBackbuffer:
	ld 	a, (m_num_drawableEntities)
	ld 	ix, #drawableEntities_vector
	ld 	hl, #rend_clearEntity
	ld 	bc, #k_dEntity_size
	call 	ent_doForAll

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CAMBIA EL BUFFER PARA LA MEMORIA DE VIDEO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_switchBuffers:
	ld 	a, (m_back_buffer)
	ld 	b, a
	ld 	a, (m_front_buffer)
	ld 	(m_back_buffer), a
	ld 	a, b
	ld 	(m_front_buffer), a

	srl 	b
	srl 	b 
	ld 		l, b
	call 	cpct_setVideoMemoryPage_asm

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DIBUJAR UNA ENTIDAD
;; REGISTROS DESTRUIDOS: AF, BC, DE, HL
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_clearEntity:
	ld 	a, (m_back_buffer)
	ld 	d, a
	ld 	e, #0

	ld 	c, de_x(ix)
	ld 	b, de_y(ix)
	call 	cpct_getScreenPtr_asm

	ex 	de, hl   		;; DE = Puntero a memoria
   ld 	a, #0			   ;; Color
   ld  	b, de_h(ix)    ;; alto
   ld  	c, de_w(ix)    ;; Ancho

   call 	cpct_drawSolidBox_asm

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DIBUJAR UNA ENTIDAD
;; REGISTROS DESTRUIDOS: AF, BC, DE, HL
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_drawEntity:
	ld 	a, (m_back_buffer)
	ld 	d, a 
	ld 	e, #0
   ld    c, de_x(ix)         		;; C = Entity Y
   ld    b, de_y(ix)         		;; B = Entity X
   call 	cpct_getScreenPtr_asm

   ex 	de, hl
   ld 	l, de_sp_l(ix)
   ld 	h, de_sp_h(ix)
   ld 	b, de_h(ix)
   ld 	c, de_w(ix)
   call 	cpct_drawSprite_asm

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DIBUJA LA VIDA DEL PERSONAJE 
;; ENTRADA:
;; 		E -> Pos en x donde dibujar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_drawHeroStats:
	ld 	hl, #_health_sp
	ld 	d, #0xC7
	ld 	bc, #0x0804
	push 	de
	call 	cpct_drawSprite_asm
	pop 	de

	ld 	hl, #_health_sp
	ld 	d, #0x87
	ld 	bc, #0x0804
	call 	cpct_drawSprite_asm

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; BORRA EL NUMERO DE OLEADA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_removeWaveNumber:
	ld 	de, #0xD826
	ld 	a, #0xC0 		;; C0 = Color negro
	ld 	bc, #0x080C
	push 	de
	call 	cpct_drawSolidBox_asm
	pop 	de

	ld 	de, #0x9826
	ld 	a, #0xC0
	ld 	bc, #0x080C
	call 	cpct_drawSolidBox_asm

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ACTUALIZA LA VIDA DEL PERSONAJE
;; ENTRADA: 
;; 		E -> Pos en x donde dibujar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_removeHeartHero:
	ld 	d, #0xC7
	ld 	a, #0xC0 		;; C0 = Color negro
	ld 	bc, #0x0804
	push 	de
	call 	cpct_drawSolidBox_asm
	pop 	de

	ld 	d, #0x87
	ld 	a, #0xC0
	ld 	bc, #0x0804
	call 	cpct_drawSolidBox_asm

	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DIBUJA EL MAPA DEL JUEGO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rend_drawMap:
	ld 	bc, #0x1914		
	ld 	de, #_mapa_W
	ld 	hl, #_tiles_0
	call 	cpct_etm_setDrawTilemap4x8_ag_asm

	ld 	hl, #0xC000
	ld 	de, #_mapa
	call 	cpct_etm_drawTilemap4x8_ag_asm

	ld 	hl, #0x8000
	ld 	de, #_mapa
	call 	cpct_etm_drawTilemap4x8_ag_asm

	ret
